use x86_64::structures::idt::{InterruptStackFrame, PageFaultErrorCode};
use crate::interrupts::InterruptIndex;

pub mod ps2;
pub mod timer;

pub extern "x86-interrupt" fn breakpoint(sf: InterruptStackFrame){
    log::error!("EXCEPT: BREAKPOINT\n{:#?}", sf);
}

pub extern "x86-interrupt" fn double_fault(stack_frame: InterruptStackFrame, _: u64) -> ! {
    panic!("EXCEPT: DOUBLE FAULT \n{:#?}", stack_frame);
}

pub extern "x86-interrupt" fn page_fault(stack_frame: InterruptStackFrame, error_code: PageFaultErrorCode) {
    use x86_64::registers::control::Cr2;

    log::error!("EXCEPT: PAGE FAULT");
    log::debug!("Addr:  {:?}", Cr2::read());
    log::debug!("ErrCo: {:?}\n{:#?}", error_code, stack_frame);
    crate::utils::hcf();
}
