use x86_64::{instructions::port::Port, structures::idt::InterruptStackFrame};

use crate::{events::EVENTMAN, interrupts::PICS};

const PS2_PORT: u16 = 0x60;

// https://wiki.osdev.org/PS/2_Keyboard
pub extern "x86-interrupt" fn ps2_kb_int(_sf: InterruptStackFrame){
    use pc_keyboard::{layouts, HandleControl, Keyboard, ScancodeSet1};
    use spin::Mutex;

    lazy_static::lazy_static! {
        static ref KEYBOARD: Mutex<Keyboard<layouts::Us104Key, ScancodeSet1>> =
            Mutex::new(Keyboard::new(ScancodeSet1::new(),
                layouts::Us104Key, HandleControl::Ignore)
            );
    }
    
    let mut keyboard = KEYBOARD.lock();
    let mut port = Port::new(PS2_PORT);

    let scancode: u8 = unsafe { port.read() };
    if let Ok(Some(key_event)) = keyboard.add_byte(scancode) {
        EVENTMAN.lock().dispatch(&crate::events::Event::Ps2KeyEvent(Some(key_event)));
    }
    unsafe {
        PICS.lock()
            .notify_end_of_interrupt(super::InterruptIndex::Ps2Keyboard.get());
    }
}
