use core::{arch::asm, future::Future};

use alloc::{collections::{btree_set::BTreeSet, vec_deque::VecDeque}, vec::Vec};
use pc_keyboard::{KeyCode, KeyEvent, KeyState};
use spin::{Lazy, Mutex, MutexGuard};
use x86_64::instructions::port::{Port, PortGeneric, ReadWriteAccess};

use crate::{events::{Event, EVENTMAN}, scheduler::{executor::Executor, Task}};
static mut KEYS_PRESSED: Mutex<BTreeSet<KeyCode>> = Mutex::new(BTreeSet::new());
static mut KEY_BUF: VecDeque<KeyEvent> = VecDeque::new();
const KEY_BUF_MAX_LEN: usize = 32;

pub struct Keyboard;
impl Keyboard {
    pub(super) unsafe fn init() {
        EVENTMAN.lock().add_listener(crate::events::Event::Ps2KeyEvent(None), |e| {
            let Event::Ps2KeyEvent(ke) = e else {panic!()};
            if let Some(ke) = ke {
                if KEY_BUF.len() > KEY_BUF_MAX_LEN {
                    KEY_BUF.pop_front();
                }
                KEY_BUF.push_back(ke.clone());
            }

            Ok(())
        });
    }
    
    pub fn keys_pressed<'a>() -> MutexGuard<'a, BTreeSet<KeyCode>> {
        unsafe { KEYS_PRESSED.lock() }
    }
    // TODO: It is possible for `KEY_BUF` to get overfilled and lose data and make keys stuck.
    // Possibly run this every now an again somehow
    unsafe fn process_events() {
        while !KEY_BUF.is_empty() {
            if let Some(ke) = KEY_BUF.pop_front() {
                match ke.state {
                    KeyState::Down => {
                        Self::keys_pressed().insert(ke.code);
                    }
                    KeyState::Up => {
                        Self::keys_pressed().remove(&ke.code);
                    }
                    _ => ()
                }
            }
        }
    }

    
    pub fn is_key_down(key: KeyCode) -> bool {
        unsafe {
            Self::process_events();
            return Self::keys_pressed().contains(&key);
        }
    }

    pub fn are_keys_down(keys: &[KeyCode]) -> bool {
        for key in keys {
            if !Self::keys_pressed().contains(key) {
                return false;
            }
        }
        true
    }


}

pub static MOUSE: Lazy<Mutex<Mouse>> = Lazy::new(|| Mutex::new(Mouse::new()));

pub struct Mouse {
}

impl Mouse {
    pub const fn new() -> Self {
        Self {
        }
    }
    
    fn get()  {
        let a = MOUSE.lock();
    }

    pub(super) fn init() {
    }

    fn on_move() {
        
    }


}

