use core::{any::{Any, TypeId}, marker::PhantomData, ops::Deref};

use alloc::{borrow::ToOwned, boxed::Box, vec};
use hashbrown::HashMap;


#[derive(Debug)]
pub struct TypeMap<'a> {
    map: HashMap<TypeId, Box<dyn Any>>,
    _phantom: PhantomData<&'a ()>
}

impl TypeMap<'_> {
    pub fn new() -> Self {
        Self {
            map: HashMap::new(),
            _phantom: PhantomData
        }
    }

    pub fn insert<T: 'static>(&mut self, value: T) {
        self.map.insert(TypeId::of::<T>(), Box::new(value));
    }

    pub fn get<T: 'static>(&self) -> Option<&T> {
        self.map.get(&TypeId::of::<T>()).and_then(|boxed| boxed.downcast_ref::<T>())
    }
}


impl<'a> IntoIterator for TypeMap<'a> {
    type Item = (TypeId, Box<dyn Any>);
    type IntoIter = hashbrown::hash_map::IntoIter<TypeId, Box<dyn Any>>;
    fn into_iter(self) -> Self::IntoIter {
        self.map.into_iter() 
    }
}
